/*
 * Copyright (C) 2024, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: zy-yuan1 <zhangyuanyuan1@kylinos.cn>
 *
 */
#include "appwindowlistmodel.h"
#include "windowmanagerinterface.h"
#include <QMetaEnum>
#include <QDebug>


AppWindowListModel::AppWindowListModel(QObject *parent) : QAbstractListModel(parent)
{
    connectSignals();
    loadWindows();
}

void AppWindowListModel::connectSignals()
{
    QObject::connect(WindowManagerInterface::getInstance(), &WindowManagerInterface::appWindowRemoved,
                     this, &AppWindowListModel::onAppWindowRemoved);

    QObject::connect(WindowManagerInterface::getInstance(), &WindowManagerInterface::appWindowDesktopChanged,
                     this, &AppWindowListModel::onAppWindowDesktopChanged);

    QObject::connect(WindowManagerInterface::getInstance(), &WindowManagerInterface::desktopMoved,
                     this, &AppWindowListModel::onDesktopMoved);
}

int AppWindowListModel::rowCount(const QModelIndex &parent) const
{
    return m_windowList.count();
}

QVariant AppWindowListModel::data(const QModelIndex &index, int role) const
{
    int i = index.row();
    if (i < 0 || i >= m_windowList.count()) {
        return QVariant();
    }
    Window window = m_windowList.at(i);
    switch (static_cast<Role>(role)) {
    case Role::Id:
        return window.id();
    case Role::Uuid:
        return window.uuid();
    case Role::Caption:
        return window.caption();
    case Role::X:
        return window.x();
    case Role::Y:
        return window.y();
    case Role::Width:
        return window.width();
    case Role::Height:
        return window.height();
    case Role::Minimized:
        return window.minimized();
    case Role::DesktopId:
        return window.desktopId();
    case Role::ScreenId:
        return window.screenId();
    default:
        return {};
    }
}

QHash<int, QByteArray> AppWindowListModel::roleNames() const
{
    QHash<int, QByteArray> roles = QAbstractItemModel::roleNames();
    QMetaEnum e = metaObject()->enumerator(metaObject()->indexOfEnumerator("Role"));

    for (int i = 0; i < e.keyCount(); ++i) {
        roles.insert(e.value(i), e.key(i));
    }
    return roles;
}

void AppWindowListModel::loadWindows()
{
    m_windowList.clear();
    beginResetModel();
    for (const Window &appwindow : getAllAppWindowList()) {
        m_windowList.append(appwindow);
    }
    endResetModel();
}

void AppWindowListModel::addItem(Window window)
{
    beginInsertRows(QModelIndex(), m_windowList.size(), m_windowList.size());
    m_windowList.append(window);
    endInsertRows();
}

void AppWindowListModel::onAppWindowRemoved(QString windowId)
{
    for (int index = 0; index < m_windowList.size(); ++index) {
        auto item = m_windowList.at(index);
        if (item.uuid() == windowId) {
            beginRemoveRows(QModelIndex(), index, index);
            m_windowList.removeAt(index);
            endRemoveRows();
            break;
        }
    }
}

void AppWindowListModel::onAppWindowDesktopChanged(QString windowId, int oldDesktopIndex, int newDesktopIndex)
{
    Window tmp;
    for (int i = 0; i < m_windowList.size(); ++i) {
        auto &item = m_windowList.at(i);
        if (item.uuid() == windowId) {
            tmp = m_windowList.at(i);
            tmp.setDesktopId(newDesktopIndex);
            m_windowList.replace(i, tmp);
            QModelIndex changeIndex = index(i, 0, QModelIndex());
            Q_EMIT dataChanged(changeIndex, changeIndex, QVector<int>{Role::DesktopId});
            break;
        }
    }
}

void AppWindowListModel::onDesktopMoved(int oldDesktopIndex, int newDesktopIndex)
{
    beginResetModel();
    loadWindows();
    endResetModel();
}

// TODO: remove?
QVariant AppWindowListModel::getItemData(const QModelIndex &index)
{
    int i = index.row();
    if (i >= 0 && i < m_windowList.count()) {
        return QVariant::fromValue(m_windowList.at(i));
    } else {
        return QVariant();
    }
}

QList<Window> AppWindowListModel::getAllAppWindowList() const
{
    QList<Window> allAppWindowList;
    for(int desktopIndex = 1; desktopIndex <= WindowManagerInterface::getInstance()->getNumberOfDesktops(); desktopIndex++) {
        for (int screenIndex = 0; screenIndex < WindowManagerInterface::getInstance()->logicalScreenCount(); screenIndex++) {
            QList<Window> windowList;
            windowList = WindowManagerInterface::getInstance()->createWindowList(desktopIndex, screenIndex);
            for (auto &window : windowList) {
                window.setDesktopId(desktopIndex);
                window.setScreenId(screenIndex);
            }
            std::reverse(windowList.begin(), windowList.end());
            allAppWindowList += windowList;
        }
    }
    return allAppWindowList;
}
