/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import DesktopListModel 1.0

FocusScope {
    id : root
    //虚拟桌面的各个参数
    property bool desktopHoverStatus : false
    property int desktopCaptionHeight : Math.floor(24 * scaleFactor)         // 虚拟桌面标题栏高度
    property var spacing : Math.floor(24 * scaleFactor)
    property int desktopRectWidth : desktopThumbnailWidth + 2 * workspaceBorderWidth
    property int desktopRectHeight : desktopThumbnailHeight + 2 * workspaceBorderWidth + spacingWidth + desktopCaptionHeight
    property int desktopMoveDistance : desktopRectWidth / 2

    Rectangle {
        id : desktopRect
        color : desktopAreaColor
        anchors.fill : parent
        property bool desktopTabBorderVisble : false

        MouseArea {
            id : desktopMouseArea
            anchors.fill : parent
            hoverEnabled : true
            onEntered: {
                desktopHoverStatus =  true
            }
            onExited: {
                desktopHoverStatus = false
            }
            onClicked: {
                focus = true
            }

            ListView {
                id : desktopListView
                anchors.fill : parent
                property bool initMultitask : true
                orientation : ListView.Horizontal
                layoutDirection: isLayoutLeftToRight ? Qt.LeftToRight : Qt.RightToLeft
                property int contentXoffset : isLayoutLeftToRight ? 0 : contentWidth
                model : desktopListModel
                spacing : root.spacing
                activeFocusOnTab: false
                clip : true
                keyNavigationEnabled: true
                interactive: true
                onActiveFocusChanged: {
                    if (desktopListView.activeFocus) {
                        currentIndex = 0
                    }

                    // interactive == false,透传鼠标事件,使desktopMouseArea clicked事件得以触发,退出工作区名称编辑状态
                    // interactive == true,截获鼠标事件,ListView交互生效
                    interactive = !activeFocus
                }

                Keys.onPressed: {
                    if (event.key === Qt.Key_PageUp || event.key === Qt.Key_Home) {
                        if (currentIndex === 0) {
                            return
                        }
                        currentIndex = 0
                        if (isLayoutLeftToRight && desktopListView.contentX > 0) {
                            contentMoveAnimation.to = 0
                            contentMoveAnimation.start()
                        } else if (!isLayoutLeftToRight && desktopListView.contentX + desktopListView.width < 0) {
                            contentMoveAnimation.to = - desktopListView.width
                            contentMoveAnimation.start()
                        }
                    } else if (event.key === Qt.Key_PageDown || event.key === Qt.Key_End) {
                        if (currentIndex === desktopListModel.rowCount() - 2) {
                            return
                        }
                        currentIndex = desktopListModel.rowCount() - 2
                        if (isLayoutLeftToRight && desktopListView.contentX  < desktopListView.contentWidth - desktopListView.width) {
                            contentMoveAnimation.to = desktopListView.contentWidth - desktopListView.width
                            contentMoveAnimation.start()
                        } else if (!isLayoutLeftToRight && desktopListView.contentX + desktopListView.contentWidth > 0) {
                            contentMoveAnimation.to = - desktopListView.contentWidth
                            contentMoveAnimation.start()
                        }
                    }

                    if (event.key === Qt.Key_Tab || event.key === Qt.Key_Left ||
                            event.key === Qt.Key_Right || event.key === Qt.Key_Up ||
                            event.key === Qt.Key_Down || event.key === Qt.Key_Home ||
                            event.key === Qt.Key_End || event.key === Qt.Key_PageUp ||
                            event.key === Qt.Key_PageDown) {
                        desktopRect.desktopTabBorderVisble = true;
                        event.accepted = false;
                    }
                }

                //这里实现了虚拟桌面listview的居中
                NumberAnimation {
                    id : desktopListViewLeftMarginAni
                    target : desktopListView
                    property : isLayoutLeftToRight ? "leftMargin" : "rightMargin"
                    duration : 400
                    easing.type : Easing.InOutQuad
                }

                populate: Transition {
                    NumberAnimation {
                        properties : "scale,opacity"
                        from : 0
                        to : 1
                        duration : 400
                        easing.type : Easing.InOutQuad
                    }
                }

                add : Transition {
                    NumberAnimation {
                        properties : "scale,opacity"
                        from : 0
                        to : 1
                        duration : 400
                        easing.type : Easing.InOutQuad
                    }
                }
                remove : Transition {
                    NumberAnimation {
                        properties : "scale,opacity"
                        from : 1
                        to : 0
                        duration : 400
                        easing.type : Easing.InOutQuad
                    }
                }
                removeDisplaced : Transition {
                    NumberAnimation {
                        properties : "x,y"
                        duration : 400
                    }
                }

                move : Transition {
                    NumberAnimation {
                        property : "x"
                        duration : 200
                        easing.type : Easing.InOutQuad
                    }
                }
                moveDisplaced : Transition {
                    NumberAnimation {
                        property : "x"
                        duration : 200
                        easing.type : Easing.InOutQuad
                    }
                }

                onCountChanged : {
                    if (initMultitask) {
                        if (count === desktopListModel.rowCount()) {
                            initMultitask = false;
                        }
                        return;
                    }

                    var newWidth = desktopRectWidth * count + desktopListView.spacing * (count - 1)
                    desktopListViewLeftMarginAni.to = (rootWindow.width - newWidth) / 2 < 0 ? 0  : (rootWindow.width - newWidth) / 2
                    desktopListViewLeftMarginAni.start()
                }

                function getWorkspaceItemOrNewWorkspaceItem(type) {
                    if(type === "newDesktop") {
                        return "NewDesktopButton.qml"
                    }
                    return "VirtualDesktopWindow.qml"
                }

                NumberAnimation {
                    id : moveToScrollEndAnimation
                    target : desktopListView
                    property : "contentX"
                    from : desktopListView.contentX
                    to : isLayoutLeftToRight ? desktopListView.contentWidth + desktopRectWidth + root.spacing - desktopListView.width :
                                               - desktopListView.contentWidth - desktopRectWidth - root.spacing
                    duration : 300
                    easing.type : Easing.InOutQuart
                }

                function moveToScrollEnd() {
                    if (desktopListView.contentWidth + desktopRectWidth + root.spacing > desktopListView.width
                            && desktopListView.count < 16) {
                        moveToScrollEndAnimation.restart()
                    }
                }

                Connections {
                    target: desktopListModel
                    function onDesktopAppended(desktopIndex) {
                        desktopListView.moveToScrollEnd(desktopIndex)
                    }
                }

                delegate: Loader {
                    source : desktopListView.getWorkspaceItemOrNewWorkspaceItem(model.ItemType)
                    onLoaded: {
                        if ("customName" in item) {
                            item.customName = model.CustomName
                        }
                    }
                    Connections {
                        target: item
                        function onCustomNameChanged() {
                            if (model.CustomName !== item.customName) {
                                desktopListModel.setData(desktopListModel.index(index, 0),
                                                         item.customName,
                                                         DesktopListModel.CustomName)
                            }
                        }
                    }
                }
            } // desktopListView

            NumberAnimation {
                id : contentMoveAnimation
                target : desktopListView
                property : "contentX"
                from : desktopListView.contentX
                duration : 300
                easing.type : Easing.InOutQuart
            }

            ArrowButton {
                id : leftButton
                visible : desktopHoverStatus && desktopListView.contentX + desktopListView.contentXoffset > 0
                x : Math.floor(48 * scaleFactor)
                y : (parent.height-leftButton.height)/2
                direction: "left"
            }

            ArrowButton {
                id : rightButton
                visible : desktopHoverStatus && desktopListView.contentX + desktopListView.contentXoffset < desktopListView.contentWidth-desktopListView.width
                x : parent.width - Math.floor(48 * scaleFactor) - rightButton.width
                y : (parent.height-rightButton.height)/2
                direction: "right"
            }
        }
    }
}
