/*
* Copyright (C) 2024, KylinSoft Co., Ltd.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*
* Authors: zy-yuan1 <zhangyuanyuan1@kylinos.cn>
*
*/

import QtQuick 2.15
import QtQuick.Controls 2.5
import org.ukui.quick.platform 1.0 as Platform

Rectangle {
    id: rootItem
    property bool editable: false
    property bool enableLayoutMirroring: false
    property color themeColor: Platform.Theme.highlight()
    property string defaultText: ""
    property string itemDisplayText: defaultText
    property real fontPointSize: Platform.Theme.fontSize
    property string fontFamily: Platform.Theme.fontFamily
    property color textColor: Platform.Theme.text()
    property int contentTextWidth: textDisplay.contentWidth
    property bool isDefaultText : false

    signal textEditingFinished()

    implicitWidth: 100
    implicitHeight: 36
    color: "transparent"
    border.color: editable ? themeColor : "transparent"
    border.width : 2
    radius: 4

    MouseArea {
        id: rootMouseArea
        anchors.fill: parent
        hoverEnabled: true
        cursorShape: editable ? Qt.IBeamCursor : Qt.ArrowCursor
        onClicked: {
            if (!editable) {
                // 点击可进入编辑状态
                editable = true;
            }
        }
    }

    onEditableChanged: {
        if (editable) {
            // force active
            textInput.forceActiveFocus();
            textInput.selectAll();
        }
        // 更新Text显示文本
        updateItemDisplayText();
        textInput.text = itemDisplayText
    }

    TextInput {
        id: textInput
        anchors.fill: parent
        anchors.centerIn : parent
        anchors.leftMargin : 4
        anchors.rightMargin : 4
        horizontalAlignment: TextInput.AlignLeft
        verticalAlignment: TextInput.AlignVCenter
        LayoutMirroring.enabled: enableLayoutMirroring

        clip: true
        maximumLength : 32
        focus: true
        selectByMouse: true
        selectionColor: themeColor
        visible: editable

        text: itemDisplayText
        font.pointSize: fontPointSize
        font.family : fontFamily
        color: textColor

        onEditingFinished: {
            editable = false;
            textEditingFinished()
        }
    }

    Text {
        id: textDisplay
        anchors.fill: parent
        anchors.centerIn: parent
        horizontalAlignment: Text.AlignLeft
        LayoutMirroring.enabled: enableLayoutMirroring
        verticalAlignment: Text.AlignVCenter

        text: itemDisplayText
        elide: enableLayoutMirroring ? Text.ElideLeft : Text.ElideRight
        visible: !editable

        font.pointSize: fontPointSize
        font.family : fontFamily
        color: textColor

        ToolTip.text: text
        ToolTip.delay: 400
        ToolTip.visible: rootMouseArea.containsMouse && textDisplay.truncated
    }

    function updateItemDisplayText() {
        itemDisplayText = textInput.length > 0 ? textInput.displayText : rootItem.defaultText
        isDefaultText = textInput.length > 0 ? false : true
    }
}
