/*
 * Copyright 2024 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "multitaskviewmodelbywlcomhandler.h"
#include <algorithm>

#include <QtDBus>
#include <QGSettings>
#include <QGuiApplication>
#include <QScreen>
#include <QtX11Extras/QX11Info>
#include <QString>
#include <QEventLoop>
#include <qpa/qplatformnativeinterface.h>
#include <iostream>
#include "multitaskviewmanagerbywlcom.h"
using namespace std;

MultitaskViewModelByWlcomHandler::MultitaskViewModelByWlcomHandler(QObject *parent) : MultitaskViewAbstractModelHandler{}
{
    createContext();
    m_context->start();
    connectSignals();
}

MultitaskViewModelByWlcomHandler::~MultitaskViewModelByWlcomHandler()
{
    delete m_context;
}

void MultitaskViewModelByWlcomHandler::createContext()
{
    struct wl_display *display = NULL;

    QPlatformNativeInterface *native = QGuiApplication::platformNativeInterface();
    if (!native) {
        return;
    }

    display = reinterpret_cast<wl_display *>(
                native->nativeResourceForIntegration(QByteArrayLiteral("wl_display")));
    if (!display) {
        qWarning("Failed to get Wayland display.");
        return;
    }

    m_context = new Context(nullptr, Context::Capability::Output | Context::Capability::Toplevel |
                          Context::Capability::Workspace);
    connect(m_context, &Context::workespaceAdded, this, &MultitaskViewModelByWlcomHandler::onWorkSpaceAdded);
    connect(m_context, &Context::toplevelAdded, this, &MultitaskViewModelByWlcomHandler::onWindowAdded);
    connect(m_context, &Context::outputAdded, this, &MultitaskViewModelByWlcomHandler::onOutputAdded);
}

Toplevel *MultitaskViewModelByWlcomHandler::getToplevel(QString windowId)
{
    Toplevel *toplevel = NULL;

    const auto wnds = m_allWindowList;
    for (auto const wnd: wnds) {
        if ((QString)wnd->uuid() == windowId) {
            toplevel = wnd;
        }
    }
    return toplevel;
}

void MultitaskViewModelByWlcomHandler::removeAppWindow(QString windowId)
{
    Toplevel *toplevel =  getToplevel(windowId);
    if (toplevel == nullptr){
        return;
    }
    toplevel->close();
}

void MultitaskViewModelByWlcomHandler::moveAppWindowToDesktop(QString windowId,
                                                              int srcDesktopInex,
                                                              int destDesktopInex)
{

    if (destDesktopInex -1 >= m_workspaceList.size()){
        return;
    }
    Toplevel *toplevel =  getToplevel(windowId);
    toplevel->moveToWorkspace(m_workspaceList[destDesktopInex-1]->uuid());

}

void MultitaskViewModelByWlcomHandler::activateAppWindow(QString windowId)
{
    getToplevel(windowId)->setActivate();
}

QIcon MultitaskViewModelByWlcomHandler::getWindowIcon(QString windowId)
{
    QIcon m_icon;
    return QIcon::fromTheme(getToplevel(windowId)->icon(), m_icon);
}

int MultitaskViewModelByWlcomHandler::getNumberOfDesktops()
{
    return m_workspaceList.size();
}

void MultitaskViewModelByWlcomHandler::removeDesktop(int desktopIndex)
{
    m_workspaceList[desktopIndex-1]->remove();
}

void MultitaskViewModelByWlcomHandler::moveDesktop(int srcDesktopIndex, int destDesktopIndex)
{
    m_workspaceList[srcDesktopIndex-1]->move(destDesktopIndex - 1);
    std::swap(m_workspaceList[srcDesktopIndex - 1], m_workspaceList[destDesktopIndex - 1]);
    emit desktopMoved(srcDesktopIndex, destDesktopIndex);
}

void MultitaskViewModelByWlcomHandler::appendDesktop()
{
    int desktopCount = m_workspaceList.size();
    m_context->addWorkspace(desktopCount);
    QEventLoop loop;
    connect(this, &MultitaskViewModelByWlcomHandler::workspaceHasCreated, &loop, &QEventLoop::quit);
    loop.exec();
}

int MultitaskViewModelByWlcomHandler::getCurrentDesktop()
{
    for (int i = 0; i < m_workspaceList.size(); i++) {
        if (m_workspaceList[i]->isActivated()) {
            return i+1;
        }
    }
    return 1;
}

bool MultitaskViewModelByWlcomHandler::isCurrentDesktopShowingDesktop()
{
    //wlcom 内部已实现，不需要获取该状态
    return false;
}

void MultitaskViewModelByWlcomHandler::setCurrentDesktop(int desktopIndex)
{
    m_workspaceList[desktopIndex-1]->setActivate();
}

QRect MultitaskViewModelByWlcomHandler::getFullScreenGeometry(int desktopIndex, int screenIndex)
{
    return QGuiApplication::screens().at(screenIndex)->geometry();
}

bool MultitaskViewModelByWlcomHandler::hasModalChildWindow(QString windowId)
{
    uint xid = windowId.toInt();
    if (KWindowSystem::hasWId(xid)) {
        KWindowInfo info(xid, NET::WMState);
        return info.hasState(NET::Modal);
    }
    return false;
}

bool MultitaskViewModelByWlcomHandler::isExtensionMode()
{  
    QRect bounding;
    for (int i = 0; i < QGuiApplication::screens().size(); i++) {
        bounding = bounding.united(QGuiApplication::screens().at(i)->geometry());
    }
    for (int i = 0; i < QGuiApplication::screens().size(); i++) {
        if (bounding == QGuiApplication::screens().at(i)->geometry()) {
            return false;
        };
    }

    return true;
}

int MultitaskViewModelByWlcomHandler::logicalScreenCount()
{
    const auto screenCount = QGuiApplication::screens().size();
    if (screenCount == 1 || !isExtensionMode()) {
        return 1;
    }
    return screenCount;
}


QList<Window> MultitaskViewModelByWlcomHandler::createWindowList(int desktopIndex, int screenIndex)
{
    QList<Window> windowList;

    const auto wnds = m_allWindowList;

    for (auto const wnd: wnds) {
        if (!shouldAcceptWindow(wnd)) {
            continue;
        }
        bool isOnDesktop = false;
        for (int i = 0; i < wnd->workspaces().size(); i++) {
            if (wnd->workspaces()[i] == (QString)m_workspaceList[desktopIndex - 1]->uuid()){
                isOnDesktop = true;
                break;
            }
        }
        if (!isOnDesktop){
            continue;
        }
        QRect wndrect(wnd->point().x(),
                      wnd->point().y(),
                      wnd->size().width(),
                      wnd->size().height());
        if (!(QGuiApplication::screens().at(screenIndex)->geometry().contains(wndrect.center()))) {
            continue;
        }
        QString id =wnd->uuid();
        Window window(id.toInt(),
                      wnd->uuid(),
                      wnd->title(),
                      wnd->point().x(),
                      wnd->point().y(),
                      wnd->size().width(),
                      wnd->size().height(),
                      wnd->isMinimized());

        windowList.append(window);

    }
    return windowList;
}

bool MultitaskViewModelByWlcomHandler::shouldAcceptWindow(Toplevel *toplevel)
{
    return !(toplevel->capabilities() & Toplevel::Capability::Switcher);
}

QString MultitaskViewModelByWlcomHandler::getDesktopUuid(int desktopIndex)
{
   return m_workspaceList[desktopIndex - 1]->uuid();
}

QString MultitaskViewModelByWlcomHandler::getOutputUuid(int outputIndex)
{
   return m_outputList[outputIndex]->uuid();
}


void MultitaskViewModelByWlcomHandler::onWindowAdded(Toplevel *toplevel)
{
    if (!MultitaskViewManagerByWlcom::isOpen) {
        if (!shouldAcceptWindow(toplevel)) {
            return;
        }
        m_allWindowList.push_back(toplevel);
        connect(toplevel, &Toplevel::stateUpdated, this, &MultitaskViewModelByWlcomHandler::onWindowUpdate);
        connect(toplevel, &Toplevel::deleted, this, &MultitaskViewModelByWlcomHandler::onWindowDeleted);
    } else if (toplevel->title() != "ukui-window-switch-wlcom") {
        emit appWindowCreated();
    }
}

void MultitaskViewModelByWlcomHandler::onOutputAdded(Output *output)
{
    m_outputList.push_back(output);
}

void MultitaskViewModelByWlcomHandler::onWindowDeleted()
{
    Toplevel *toplevel = qobject_cast<Toplevel *>(sender());
    for (int i = 0; i < m_allWindowList.size(); i++){
        if (m_allWindowList[i]->uuid() == toplevel->uuid()){
            m_allWindowList.removeAt(i);
            break;
        }
    }
    QString id = toplevel->uuid();

    qDebug()<<"toplevel"<<id;
    emit appWindowRemoved(id);
}

void MultitaskViewModelByWlcomHandler::onWindowActivated(Toplevel *toplevel)
{
    if (!shouldAcceptWindow(toplevel)) {
        return;
    }
    // KWin在发送windowActivated信号时
    // window对象是有可能为空的。对于多任务
    // 视图来说，只关心真正有窗口被激活的清空。
    // 因此，如果window对象为空，则不转发
    // 该信号
    if (toplevel->appId() == nullptr) {
        return;
    }
    if (toplevel->isActivated()) {
        int index = m_allWindowList.indexOf(toplevel);
        if (index != -1) {
            m_allWindowList.move(index, m_allWindowList.size() - 1);
        }
        emit appWindowActivated();
    }
}

void MultitaskViewModelByWlcomHandler::onWindowUpdate(Toplevel::Masks mask)
{
    Toplevel *toplevel = qobject_cast<Toplevel *>(sender());

    if (mask & Toplevel::Mask::Activated) {
        onWindowActivated(toplevel);
    }

    if (mask & Toplevel::Mask::Workspace) {
        QStringList workspaces = toplevel->workspaces();
        if (!workspaces.isEmpty()) {
            for (int i = 0; i <m_workspaceList.size(); i++) {
                if (m_workspaceList[i]->uuid() == workspaces.at(0)) {
                    onDesktopPresenceChanged(toplevel,0,i+1);
                    break;
                }
            }
        }
    }

}

void MultitaskViewModelByWlcomHandler::onDesktopPresenceChanged(Toplevel *toplevel, int oldDesktopIndex ,int newDesktopIndex)
{
    if (!shouldAcceptWindow(toplevel)) {
        return;
    }
    for (int i = 0; i < m_allWindowList.size(); i++){
        if (m_allWindowList[i]->uuid() == toplevel->uuid()){
            m_allWindowList[i] = toplevel;
            break;
        }
    }
    emit appWindowDesktopChanged(toplevel->uuid(), oldDesktopIndex, newDesktopIndex);
}


void MultitaskViewModelByWlcomHandler::onWorkSpaceAdded(Workspace *workspace)
{
    m_workspaceList.push_back(workspace);
    connect(workspace, &Workspace::stateUpdated, this, &MultitaskViewModelByWlcomHandler::onCurrentDesktopChanged);
    connect(workspace, &Workspace::deleted, this, &MultitaskViewModelByWlcomHandler::onWorkSpaceDeleted);
    emit workspaceHasCreated();
    emit desktopAppended(m_workspaceList.size());
}

void MultitaskViewModelByWlcomHandler::onWorkSpaceDeleted()
{
    Workspace *workspace = qobject_cast<Workspace *>(sender());
    for (int i = 0; i < m_workspaceList.size(); i++) {
        if (workspace->uuid() == (QString)m_workspaceList[i]->uuid()){
            m_workspaceList.removeAt(i);
            emit desktopRemoved(i+1);
            break;
        }
    }
}

void MultitaskViewModelByWlcomHandler::onCurrentDesktopChanged(Workspace::Masks mask)
{
    Workspace *workspace = qobject_cast<Workspace *>(sender());

    if (mask & Workspace::Mask::Position) {
        if (workspace->isActivated()){
            for (int i = 0; i < m_workspaceList.size(); i++) {
                if (workspace->uuid() == m_workspaceList[i]->uuid()){
                    m_workspaceList[i] = workspace;
                    emit currentDesktopChanged(0,i+1);
                    break;
                }
            }
        }
    }

    if (mask & Workspace::Mask::Activated) {
        if (workspace->isActivated()){
            for (int i = 0; i < m_workspaceList.size(); i++) {
                if (workspace->uuid() == m_workspaceList[i]->uuid()){
                    m_workspaceList[i] = workspace;
                    emit currentDesktopChanged(0,i+1);
                    break;
                }
            }
        }
    }

}


void MultitaskViewModelByWlcomHandler::connectSignals()
{
    //    QObject::connect(KWindowSystem::self(), &KWindowSystem::windowAdded,
    //                     this, &MultitaskViewModelByWlcomHandler::onWindowAdded);
    //    QObject::connect(KWindowSystem::self(), &KWindowSystem::windowRemoved,
    //                     this, &MultitaskViewModelByWlcomHandler::onWindowDeleted);
    //    QObject::connect(KWindowSystem::self(), &KWindowSystem::activeWindowChanged,
    //                     this, &MultitaskViewModelByWlcomHandler::onWindowActivated);
    //    QObject::connect(KWindowSystem::self(), static_cast<void (KWindowSystem::*)(WId, NET::Properties, NET::Properties2)>(&KWindowSystem::windowChanged),
    //                     this, &MultitaskViewModelByWlcomHandler::onDesktopPresenceChanged);
    //    QObject::connect(KWindowSystem::self(), &KWindowSystem::currentDesktopChanged,
    //                     this, &MultitaskViewModelByWlcomHandler::onCurrentDesktopChanged);
    //    QObject::connect(KWindowSystem::self(), SIGNAL(numberOfDesktopsChanged(int)),
    //                     this, SIGNAL(desktopNumberChanged()));
}
