/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "windowmanagerinterface.h"

#include <algorithm>

#include <QGSettings>
#include <QSettings>
#include <QGuiApplication>
#include <QScreen>
#include <QDBusConnection>
#include <QDBusMessage>
#include <QFile>
#include <QJsonDocument>
#include <QJsonObject>
#include <QDir>
#include <mutex>
#ifdef KWINEFFECT
#include "ukui-window-switch-kwineffect/multitaskviewmodelbycompositehandler.h"
#endif
#include "ukui-window-switch-wlcom/multitaskviewmodelbywlcomhandler.h"

static const QString UKUI_STYLE_SCHEMA = "org.ukui.style";
static const QString UKUI_STYLE_SCHEMA_STYLENAME = "styleName";
static const QString UKUI_STYLE_SCHEMA_FONTSIZE = "systemFontSize";
static const QString UKUI_STYLE_SCHEMA_FONT = "systemFont";
static const QString UKUI_STYLE_SCHEMA_WIDGETNAME = "widgetThemeName";
const QString IGNORE_WINDOW_CONFIG = "/usr/share/ukui/ukui-panel/panel-commission.ini";
static const QString DESKTOP_NAME = "desktop-name";
const QString DESKTOP_CONFIG_DIR = QDir::homePath() + QStringLiteral("/.config/org.ukui/ukui-window-switch/");
const QString DESKTOP_CONFIG = DESKTOP_CONFIG_DIR + QStringLiteral("multitaskview.conf");

WindowManagerInterface *WindowManagerInterface::getInstance()
{
    static WindowManagerInterface s_interface;
    return &s_interface;
}

WindowManagerInterface::WindowManagerInterface(QObject *parent) : QObject(parent)
{
    //kwineffect相关接口无法在非effect插件的环境下编译通过，需要通过预编译来进行判断
#ifdef KWINEFFECT
    m_modelhandler = new MultitaskViewModelByCompositeHandler(this);
#else
    m_modelhandler = new MultitaskViewModelByWlcomHandler(this);
#endif

    updateInputMode();
    connectSignals();
    if (QGSettings::isSchemaInstalled(UKUI_STYLE_SCHEMA.toUtf8())) {
        m_settings = new QGSettings(UKUI_STYLE_SCHEMA.toUtf8());
    }
}

WindowManagerInterface::~WindowManagerInterface()
{
    if (m_modelhandler != nullptr) {
        delete m_modelhandler;
        m_modelhandler = nullptr;
    }
}

bool WindowManagerInterface::currentDesktopHasWindow() const
{
    for (int screenIndex = 0; screenIndex < logicalScreenCount(); screenIndex++) {
        if (m_modelhandler->createWindowList(getCurrentDesktop(), screenIndex).size() > 0) {
            return true;
        }
    }
    return false;
}

QString WindowManagerInterface::getDesktopUuid(int desktopIndex)
{
    if (!validateDesktopIndex(desktopIndex)) {
        return QString();
    }
    return m_modelhandler->getDesktopUuid(desktopIndex);
}

QString WindowManagerInterface::getOutputUuid(int outputIndex)
{
    return m_modelhandler->getOutputUuid(outputIndex);
}


void WindowManagerInterface::removeAppWindow(QString windowId)
{
    m_modelhandler->removeAppWindow(windowId);
}

void WindowManagerInterface::removeAllAppWindowOnDesktop(int desktopIndex, int screenIndex)
{
    QList<Window> windowList;
    windowList = m_modelhandler->createWindowList(desktopIndex, screenIndex);
    for (Window &window : windowList) {
        m_modelhandler->removeAppWindow(window.uuid());
    }
}

int WindowManagerInterface::getNumberOfDesktops() const
{
    return m_modelhandler->getNumberOfDesktops();
}

void WindowManagerInterface::moveAppWindowToDesktop(QString windowId, int srcDesktopInex, int destDesktopInex) const
{
    m_modelhandler->moveAppWindowToDesktop(windowId, srcDesktopInex, destDesktopInex);
}

void WindowManagerInterface::activateAppWindow(QString windowId)
{
    m_modelhandler->activateAppWindow(windowId);
}

QIcon WindowManagerInterface::getWindowIcon(QString windowId)
{
    return m_modelhandler->getWindowIcon(windowId);
}

void WindowManagerInterface::removeDesktop(int desktopIndex)
{
    if (m_modelhandler->getNumberOfDesktops() <= 1) {
        return;
    }
    if (!validateDesktopIndex(desktopIndex)) {
        return;
    }
    m_modelhandler->removeDesktop(desktopIndex);
}

void WindowManagerInterface::appendDesktop()
{
    int MAX_DESKTOP_COUNT = 16;
    if (m_modelhandler->getNumberOfDesktops() >= MAX_DESKTOP_COUNT) {
        return;
    }
    m_modelhandler->appendDesktop();
}

void WindowManagerInterface::moveDesktop(int srcDesktopIndex, int destDesktopIndex)
{
    if (srcDesktopIndex == destDesktopIndex) {
        return;
    }
    m_modelhandler->moveDesktop(srcDesktopIndex, destDesktopIndex);
}

QList<Window> WindowManagerInterface::createWindowList(int desktopIndex, int screenIndex)
{
    return m_modelhandler->createWindowList(desktopIndex, screenIndex);
}

int WindowManagerInterface::getCurrentDesktop() const
{
    return m_modelhandler->getCurrentDesktop();
}

bool WindowManagerInterface::isCurrentDesktopShowingDesktop() const
{
    return m_modelhandler->isCurrentDesktopShowingDesktop();
}

void WindowManagerInterface::setCurrentDesktop(int desktopIndex)
{
    if (!validateDesktopIndex(desktopIndex)) {
        return;
    }
    m_modelhandler->setCurrentDesktop(desktopIndex);
}

QRect WindowManagerInterface::getFullScreenGeometry(int desktopIndex, int screenIndex) const
{
    return m_modelhandler->getFullScreenGeometry(desktopIndex, screenIndex);
}

bool WindowManagerInterface::hasModalChildWindow(QString windowId)
{
    return m_modelhandler->hasModalChildWindow(windowId);
}


QString WindowManagerInterface::getUKUIStyleName() const
{
    // default to ukui-default
    QString styleName = "ukui-default";
    if (m_settings && m_settings->keys().contains(UKUI_STYLE_SCHEMA_STYLENAME)) {
        styleName = m_settings->get(UKUI_STYLE_SCHEMA_STYLENAME).toString();
    } else {
        qWarning() << QString("未查询到Gesetting的键值(%1)").arg(UKUI_STYLE_SCHEMA_STYLENAME);
    }
    return styleName;
}

double WindowManagerInterface::getUKUIFontSize() const
{
    // default to 11
    double fontSize = 11;
    if (m_settings && m_settings->keys().contains(UKUI_STYLE_SCHEMA_FONTSIZE)) {
        fontSize = m_settings->get(UKUI_STYLE_SCHEMA_FONTSIZE).toDouble();
    } else {
        qWarning() << QString("未查询到Gesetting的键值(%1)").arg(UKUI_STYLE_SCHEMA_FONTSIZE);
    }
    return fontSize;
}

QString WindowManagerInterface::getUKUIFont() const
{
    // default to 11
    QString font = "Noto Sans CJK SC";
    if (m_settings && m_settings->keys().contains(UKUI_STYLE_SCHEMA_FONT)) {
        font = m_settings->get(UKUI_STYLE_SCHEMA_FONT).toString();
    } else {
        qWarning() << QString("未查询到Gesetting的键值(%1)").arg(UKUI_STYLE_SCHEMA_FONT);
    }
    return font;
}

void WindowManagerInterface::setIsOpenGLBackend(bool isOpenGLBackend)
{
    m_isOpenGLBackend = isOpenGLBackend;
}

int WindowManagerInterface::getRadius(QString type) const
{
    return m_isOpenGLBackend ? getUKUIRadius(type) : 0;
}

int WindowManagerInterface::getUKUIRadius(QString type) const
{
    // default to 8
    QString widgetName = "default";
    if (m_settings && m_settings->keys().contains(UKUI_STYLE_SCHEMA_WIDGETNAME)) {
        widgetName = m_settings->get(UKUI_STYLE_SCHEMA_WIDGETNAME).toString();
    } else {
        qWarning() << QString("未查询到Gesetting的键值(%1)").arg(UKUI_STYLE_SCHEMA_WIDGETNAME);
        return 8;
    }

    QFile file(QString("/usr/share/qt5-ukui-platformtheme/themeconfig/%1.json").arg(widgetName));
    if (!file.open(QIODevice::ReadOnly)) {
        qWarning() << "无法打开JSON文件";
        return 8;
    }

    QByteArray jsonData = file.readAll();
    file.close();

    // 将JSON数据转换为QJsonDocument对象
    QJsonParseError error;
    QJsonDocument jsonDoc = QJsonDocument::fromJson(jsonData, &error);
    if (error.error != QJsonParseError::NoError) {
        qWarning() << "JSON格式错误:" << error.errorString();
        return 8;
    }

    // 获取属性
    QJsonObject rootObj = jsonDoc.object();
    QJsonObject radiusObj = rootObj["Radius"].toObject();
    QJsonObject radiusValueObj = radiusObj[type].toObject();

    return radiusValueObj["value"].toString().toInt();
}

bool WindowManagerInterface::isExtensionMode()
{
    return m_modelhandler->isExtensionMode();
}

int WindowManagerInterface::logicalScreenCount() const
{
    return m_modelhandler->logicalScreenCount();
}

bool WindowManagerInterface::isTabletMode() const
{
    // default to PC mode.
    bool tabletMode = false;
    QDBusMessage message = QDBusMessage::createMethodCall("com.kylin.statusmanager.interface",
                                                          "/",
                                                          "com.kylin.statusmanager.interface",
                                                          "get_current_tabletmode");
    QDBusMessage response = QDBusConnection::sessionBus().call(message);
    if (response.type() == QDBusMessage::ReplyMessage) {
        tabletMode = response.arguments().takeFirst().toBool();
    } else {
        qDebug() << "get tablet mode failed " << response.type();
    }
    return tabletMode;
}

QStringList WindowManagerInterface::getDesktopNames() const
{
    QSettings desktopSettings(DESKTOP_CONFIG, QSettings::IniFormat);
    desktopSettings.setIniCodec("utf-8");
    QStringList nameList = desktopSettings.value("desktops/name").toStringList();
    return nameList;
}

void WindowManagerInterface::saveDesktopNames(const QStringList &nameList)
{
    QDir configDir;
    if (!configDir.exists(DESKTOP_CONFIG_DIR)) {
        configDir.mkdir(DESKTOP_CONFIG_DIR);
    }

    QSettings desktopSettings(DESKTOP_CONFIG, QSettings::IniFormat);
    desktopSettings.setIniCodec("utf-8");
    desktopSettings.beginGroup("desktops");
    desktopSettings.setValue("name", QVariant(nameList));
    desktopSettings.endGroup();
    //qDebug() << "[MultitaskViewModel] save custom desktop names" << desktopSettings.value("desktops/name").toStringList();
}

QString WindowManagerInterface::getCurrentLanguage() const
{
    // default to "Chinese".
    QLocale locale = QLocale::system();
    QString language = "Chinese";
    if (locale.language() == QLocale::Chinese) {
        language = "Chinese";
    } else if (locale.language() == QLocale::Tibetan) {
        language = "Tibetan";
    }
    return language;
}

void WindowManagerInterface::updateInputMode()
{

    QDBusMessage message = QDBusMessage::createMethodCall("com.kylin.statusmanager.interface",
                                                          "/",
                                                          "com.kylin.statusmanager.interface",
                                                          "get_inputmethod_mode");
    QDBusMessage response = QDBusConnection::sessionBus().call(message);
    if (response.type() == QDBusMessage::ReplyMessage) {
        m_inputMode = response.arguments().takeFirst().toBool();
        qDebug() << "m_inputMode =" << m_inputMode;
    } else {
        // invalid response, keep default.
    }
}

bool WindowManagerInterface::validateDesktopIndex(int desktopIndex) const
{
    return desktopIndex >= 1 && desktopIndex <= m_modelhandler->getNumberOfDesktops();
}

bool WindowManagerInterface::validateScreenIndex(int screenIndex) const
{
    return screenIndex >= 0 || screenIndex < logicalScreenCount();
}

void WindowManagerInterface::connectSignals()
{
    QDBusConnection::sessionBus().connect("com.kylin.statusmanager.interface",
                                          "/",
                                          "com.kylin.statusmanager.interface",
                                          "inputmethod_change_signal",
                                          this,
                                          SLOT(inputModeSlot(bool)));
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::tabletModeChanged,
                     this, &WindowManagerInterface::tabletModeChanged, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::currentDesktopChanged,
                     this, &WindowManagerInterface::currentDesktopChanged, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::desktopNumberChanged,
                     this, &WindowManagerInterface::desktopNumberChanged, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::desktopRemoved,
                     this, &WindowManagerInterface::desktopRemoved, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::desktopAppended,
                     this, &WindowManagerInterface::desktopAppended, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::desktopMoved,
                     this, &WindowManagerInterface::desktopMoved, Qt::QueuedConnection);

    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::appWindowCreated,
                     this, &WindowManagerInterface::appWindowCreated, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::appWindowRemoved,
                     this, &WindowManagerInterface::appWindowRemoved, Qt::QueuedConnection);
    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::appWindowDesktopChanged,
                     this, &WindowManagerInterface::appWindowDesktopChanged, Qt::QueuedConnection);

    QObject::connect(m_modelhandler, &MultitaskViewAbstractModelHandler::screenSizeChanged,
                     this, &WindowManagerInterface::screenSizeChanged, Qt::QueuedConnection);
    QObject::connect(qGuiApp, &QGuiApplication::screenAdded,
                     this, &WindowManagerInterface::screenCountChanged, Qt::QueuedConnection);
    QObject::connect(qGuiApp, &QGuiApplication::screenRemoved,
                     this, &WindowManagerInterface::screenCountChanged, Qt::QueuedConnection);
}

void WindowManagerInterface::inputModeSlot(bool inputMode)
{
    m_inputMode = inputMode;
    emit inputModeChanged(m_inputMode);
}
